@@ -1,4 +1,4 @@
-/* $Id: conns.h,v 1.4 2001/11/21 01:00:09 rjkaes Exp $
+/* $Id: conns.h,v 1.10 2002/05/23 18:23:29 rjkaes Exp $
  *
  * See 'conns.c' for a detailed description.
  *
@@ -18,28 +18,51 @@
 #ifndef TINYPROXY_CONNS_H
 #define TINYPROXY_CONNS_H
 
+#include "tinyproxy.h"
+
 /*
  * Connection Definition
  */
 struct conn_s {
 	int client_fd;
 	int server_fd;
+
 	struct buffer_s *cbuffer;
 	struct buffer_s *sbuffer;
-	bool_t simple_req;
-	bool_t ssl;
-	bool_t upstream;
-	bool_t send_message;
+
+	/* The request line (first line) from the client */
+	char *request_line;
+
+	bool_t connect_method;
+	bool_t show_stats;
+
+	/* Store the error response if there is one */
+	char *error_string;
+	int error_number;
+
+	/* A Content-Length value from the remote server */
+	long remote_content_length;
+
+	/*
+	 * Store the client's IP and hostname information
+	 */
+	char* client_ip_addr;
+	char* client_string_addr;
+
+	/*
+	 * Store the incoming request's HTTP protocol.
+	 */
 	struct {
-		unsigned short int major;
-		unsigned short int minor;
+		unsigned int major;
+		unsigned int minor;
 	} protocol;
 };
 
 /*
  * Functions for the creation and destruction of a connection structure.
  */
-extern void initialize_conn(struct conn_s *connptr);
+extern struct conn_s* initialize_conn(int client_fd, const char* ipaddr,
+				      const char* string_addr);
 extern void destroy_conn(struct conn_s *connptr);
 
 #endif
